// file: EnergyPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.net.*;

/**
 *
 * EnergyPanel is used as a frame of a EnergyPlot.
 *
 * hierarchy: JPanel -> SubPanel -> EnergyPanel
 */
public class EnergyPanel extends SubPanel {

    //**********************************************************************
    //
    // declare global variables
    //
    //**********************************************************************
    // declare the energy plot canvas
    private EnergyPlot canvas;

    // name of this panel
    private String title;
    
    // font of name label
    private Font currentFont = getFont();   // fonts for the title border
    private Font newFont = new Font(currentFont.getName(), 
				    currentFont.getStyle(), 12);
    // border of the panel
    private TitledBorder titleborder;

    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * Default class constructor
     *
     * @param str used to form the title label
     */
    EnergyPanel(String str) {

	canvas = new EnergyPlot();

	// create the title border using input argument
	title = str;
	titleborder = new TitledBorder
	    (BorderFactory.createLineBorder(Color.black),
	     title,
	     TitledBorder.LEFT,
	     TitledBorder.ABOVE_TOP,
	     newFont,
	     Color.black);

	setBorder(titleborder);
	// add canvas to the panel
	addComponents();
    }

    // *****************************************************************
    // 
    // declare class methods
    //
    // *****************************************************************

    // method: addComponents
    //
    // arguments: none
    // return   : none
    //
    // add canvas to the panel
    //
    private void addComponents() {
	constrain(this, canvas, 0, 0,
		  GridBagConstraints.REMAINDER, 1,
		  GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
		  1, 1, 0, 0, 0, 0);
    }

    /**
     * changes title label
     *
     * @param a title to be set
     */
    public void changeTitle(String a) {
	titleborder.setTitle(a);
    }

    /**
     * recover the canvas
     */
    public void recoverPanel() {
	canvas.recoverPanel();
    }

    /**
     * repaint canvas
     */
    public void reFresh() {
	super.repaint();
	canvas.repaint();
    }

    /**
     * send lpcData to canvas
     * @param d lpcData to be attached
     */
    public void attachData(lpcData d) {
	//	if (d == null) {
	    //	    canvas.DataAttached = False;
	//  return;
	//	}
	canvas.attachData(d);
    }

    /**
     * clear canvas
     */
    public void clearPanel() {
	canvas.clearPanel();
    }

    /**
     * set max value of x for canvas
     *
     * @param a int value to set xMax variable
     */
    public void setXMax(int a) {
	canvas.setXMax(a);
    }

    /**
     * set min value of x for canvas
     *
     * @param a int vallue to set xMin variable
     */
    public void setXMin(int a) {
	canvas.setXMin(a);
    }


    /**
     * set draw mode for canvas
     *
     * @param a String value of desired drawing model
     */
    public void setDrawMode(String a) {
	canvas.setDrawMode(a);
    }
}
// end of file
