// file: MainMenu.java
//

// import necessary java libraries
//
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.applet.*;
import java.applet.AudioClip;

// import local class
//
//import Constants;


/**
 * MainMenu implements a menu driven system that acts as the primary 
 * control panel for the applet as a whole.
 *
 * hierarchy: JPanel->SubPanel->MainMenu
 */
public class MainMenu extends SubPanel 
    implements ActionListener, Constants {

    // *********************************************************************
    //
    // declare private variables and components
    //
    // *********************************************************************

    // declare the menu font layout
    private Font currentFont = getFont();
    private Font newFont = new Font(currentFont.getName(), 
				    currentFont.getStyle(), 12);
    // declare objects needed by the menu sub-components
    //
    private JMenuBar menuBar;
    private JMenu menu;
    private JMenuItem menuItem;
    private JRadioButtonMenuItem rbmenuItem;
    
    // declare the string of return
    private String newline = new String("\n");
    
    // declare the audiofile URL     
    private URL audiofile = null;

    // declare the frame for accepting test signal submitted by usr
    //
    private JFrame urlframe;
    private JLabel urlmsglabel;
    private JTextField urlstring;
    private JButton urlsubmit;
    private JButton urlcancel;
    private URL signaladdress;

    // declare the frame for scaling
    //
    private JFrame scale;
    private JTextField xmax;
    private JTextField xmin;
    private JTextField ymax;
    private JTextField ymin;
    private JButton cancel;
    private JButton update;

    // declare the frame for error message
    //
    private JFrame message;
    private JTextArea errortext;
    private JButton msgdismiss;

    // declare the frame and components for the help box
    //
    private JFrame helpframe;
    private JTextArea helptext;
    private JScrollPane scrollPane;
    private JButton helpdismiss;

    // declare variables to read the help file
    //
    private URL helpfile;
    private Vector urlvec;
    private String document;
    private BufferedReader buff;
    private AppletContext context;

    // declare components to be controlled
    //
    private match gridmatch;
    private OutputPanel output;
    private ProcessBox process;
    private SignalPanel refspec;
    private SignalPanel testspec;
    private SignalPanel warpedspec;
    private WavePanel waveform;

    private EnergyPanel refenergy;
    private EnergyPanel testenergy;
    // declare the signal source from which an input signale
    // can be selected from 
    private inputSignals signalSource;

    // declare the applet in which this menu will be installed
    //
    private dtwApplet applet;

    // declare the output format for process box
    private String pattern = "###.###";
    private DecimalFormat myFormat = new DecimalFormat(pattern);


    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
   
    /**
     * default class constructor that initializes objects of the main menu
     *
     * @param gm       match object initialized
     * @param out      OutputPanel object of instantiation
     * @param pb       ProcessBox object of instantiation that gives
     *                 description of the process
     * @param specref  SignalPanel object of instantiation of model
     * @param spectest SignalPanel object of instantiation of signal to test
     * @param warp     SignalPanel object of instantiation of warped output
     * @param refey    EnergyPanel object of instantiation of energy model
     * @param testey   EnergyPanel object of instantiation of energy of
     *                 the signal
     * @param wave     WavePanel object of instantiation of test signal
     * @param insig    inputSignals that can be used
     * @param con      AppletContext to be initialized
     * @param vec      Vector of url's
     */ 
    public MainMenu (match gm,
	      OutputPanel out, 
	      ProcessBox pb, SignalPanel specref, 
	      SignalPanel spectest, SignalPanel warp, 
	      EnergyPanel refey, EnergyPanel testey,
	      WavePanel wave,
	      inputSignals insig, AppletContext con, Vector vec) {

	super();
	setBorder(BorderFactory.createEtchedBorder());

	gridmatch = gm;
	output = out;
	process = pb;
	refspec = specref;
	testspec = spectest;
	warpedspec = warp;
	signalSource = insig;
	refenergy = refey;
	testenergy = testey;
	waveform = wave;
	context = con;
	urlvec = vec;

	// create the menu bar item
	//
	menuBar = new JMenuBar();
	
	// build the template menu
	//
	menu = new JMenu("Models");
	menu.setFont(newFont);
	//	menu.getAccessibleContext().setAccessibleDescription(templateOption);
	menuBar.add(menu);

	// add the options to the template nemu
	//
	menuItem = new JMenuItem("One");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Two");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Three");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Four");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Five");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Six");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Seven");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Eight");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Nine");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new JMenuItem("Zero");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Oh");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	

	// build the signal menu
	//
	menu = new JMenu("Signal");
	menu.setFont(newFont);
	//	menu.getAccessibleContext().setAccessibleDescription(signalOption);
	menuBar.add(menu);

	// add the "from file" option
	//
	menuItem = new JMenuItem("From File");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add menu separator
	menu.addSeparator();

	menuItem = new JMenuItem("Select Signal");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menu.addSeparator();
	

	//add the options to the signal menu
	//
	menuItem = new JMenuItem("One - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Two - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Three - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Four - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Five - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Six - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Seven - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Eight - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Nine - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Zero - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Oh - Male");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// build the constraint menu
	//
	menu = new JMenu("Constraint");
	menu.setFont(newFont);
	//	menu.getAccessibleContext().setAccessibleDescription(constraintOption);
	menuBar.add(menu);

	// add a list of radio buttons to the constraint  menu
	//
	ButtonGroup group1 = new ButtonGroup();

	// add constraint to the menu
	//
	rbmenuItem = new JRadioButtonMenuItem("None");
	rbmenuItem.setFont(newFont);
	rbmenuItem.addActionListener(this);
	rbmenuItem.setSelected(true);
	group1.add(rbmenuItem);
	menu.add(rbmenuItem);

	rbmenuItem = new JRadioButtonMenuItem("Slope 2 - 1/2");
	rbmenuItem.setFont(newFont);
	rbmenuItem.addActionListener(this);
	group1.add(rbmenuItem);
	menu.add(rbmenuItem);
	
	rbmenuItem = new JRadioButtonMenuItem("Slope 3 - 1/3");
	rbmenuItem.setFont(newFont);
	rbmenuItem.addActionListener(this);
	group1.add(rbmenuItem);
	menu.add(rbmenuItem);

	// build the distance menu
	//
	menu = new JMenu("Distance");
	menu.setFont(newFont);
	menuBar.add(menu);

	// add a list of radio buttons to the distance menu
	//
	group1 = new ButtonGroup();

	// add distance to the menu
	//
	rbmenuItem = new JRadioButtonMenuItem("Itakura Distance");
	rbmenuItem.setFont(newFont);
	rbmenuItem.addActionListener(this);
	rbmenuItem.setSelected(true);
	group1.add(rbmenuItem);
	menu.add(rbmenuItem);

	rbmenuItem = new JRadioButtonMenuItem("Euclidean Distance");
	rbmenuItem.setFont(newFont);
	rbmenuItem.addActionListener(this);
	group1.add(rbmenuItem);
	menu.add(rbmenuItem);


	rbmenuItem = new JRadioButtonMenuItem("Absolute Distance");
	rbmenuItem.setFont(newFont);
	rbmenuItem.addActionListener(this);
	group1.add(rbmenuItem);
	menu.add(rbmenuItem);

	// build the go menu
	//
	menu = new JMenu("Go");
	menu.setFont(newFont);
	//	menu.getAccessibleContext().setAccessibleDescription(goOption);
	menuBar.add(menu);

	// add go options to the menu
	//
	menuItem = new JMenuItem("Compute");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add menu separator
	menu.addSeparator();

	menuItem = new JMenuItem("Recognize");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add menu separator
	menu.addSeparator();

	menuItem = new JMenuItem("Play");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// build the edit menu
	//
	menu = new JMenu("Edit");
	menu.setFont(newFont);
	//	menu.getAccessibleContext().setAccessibleDescription(editOption);
	menuBar.add(menu);

	// add scale options to the menu
	//
	menuItem = new JMenuItem("Scaling");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add menu separator
	menu.addSeparator();

	menuItem = new JMenuItem("Reset");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add menu separator
	menu.addSeparator();

	menuItem = new JMenuItem("Clear");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Clear Description");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);


	// build the Help menu
	//
	menu = new JMenu("Help");
	menu.setFont(newFont);
	//	menu.getAccessibleContext().setAccessibleDescription(helpOption);
	menuBar.add(Box.createHorizontalGlue());
	menuBar.add(menu);

	// add the Release option to the menu
	//
	menuItem = new JMenuItem("Release Notes");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add the Information option to the menu
	//
	menuItem = new JMenuItem("Information and Support");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	// add the Overview option to the menu
	//
	menuItem = new JMenuItem("Applet Overview");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// add the Tutorial option to the menu
	//
	menuItem = new JMenuItem("Applet Tutorial");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	// add the Plug-ins option to the menu
	//
	menuItem = new JMenuItem("About Plug-ins");
	menuItem.setFont(newFont);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// create URL accepting, scaling, error message, and help frames
	createURLAcceptFrame();
	createScaleFrame();
	createMessageFrame();
	createHelpFrame();

	this.addComponents();
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * adds components to the control panel
     */
    public void addComponents() {
	
	constrain(this, menuBar, 0, 0, 
		  GridBagConstraints.REMAINDER, 
		  GridBagConstraints.REMAINDER,
		  GridBagConstraints.BOTH, 
		  GridBagConstraints.CENTER,
		  1, 1, 0, 0, 0, 0);
    }

    /**
     * method create the URL accepting frame to accept the test signal URL
     * submitted by user
     */
    private void createURLAcceptFrame() {
	urlframe = new JFrame("Choose");
	Dimension dim = new Dimension(500, 120);
	urlframe.setSize(dim);
	
	urlmsglabel = new JLabel("Input the World Wide Web location (URL) of the signal:");
	dim = new Dimension(500, 40);
	urlmsglabel.setPreferredSize(dim);

	urlstring = new JTextField();
	urlstring.setFont(newFont);
	urlstring.setPreferredSize(dim);

	urlsubmit = new JButton("Submit");
	urlsubmit.addActionListener(this);
	urlsubmit.setActionCommand("URL Submit");
	dim = new Dimension(100, 40);
	urlsubmit.setPreferredSize(dim);

	urlcancel = new JButton("Cancel");
	urlcancel.addActionListener(this);
	urlcancel.setActionCommand("URL Cancel");
	dim = new Dimension(100, 40);
	urlcancel.setPreferredSize(dim);

	GridBagLayout gblayout = new GridBagLayout();
	urlframe.getContentPane().setLayout(gblayout);
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 3;
	c.gridheight = 1;
	urlframe.getContentPane().add(urlmsglabel, c);

	c.gridx = 0;
	c.gridy = 1;
	urlframe.getContentPane().add(urlstring, c);
	
	c.gridx = 1;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	urlframe.getContentPane().add(urlsubmit, c);

	c.gridx = 2;
	c.gridy = 2;
	urlframe.getContentPane().add(urlcancel, c);
    }

	
	
    /**
     * method create the scaling frame to accept the change to the canvas
     * display area
     */
    public void createScaleFrame() {
	scale = new JFrame("Scaling");
	Dimension d = new Dimension(200, 200);
	scale.setSize(d);

	JLabel Xmax = new JLabel("XMaximum");
	JLabel Xmin = new JLabel("XMinimum");
	Xmax.setFont(newFont);
	Xmin.setFont(newFont);

	xmax = new JTextField();
	xmin = new JTextField();
	xmax.setFont(newFont);
	xmin.setFont(newFont);

	cancel = new JButton("Cancel");
	update = new JButton("Update");
	cancel.setFont(newFont);
	update.setFont(newFont);
	cancel.addActionListener(this);
	update.addActionListener(this);

	GridLayout glayout = new GridLayout(3, 2);
	scale.getContentPane().setLayout(glayout);
	scale.getContentPane().add(Xmin);
	scale.getContentPane().add(xmin);
	scale.getContentPane().add(Xmax);
	scale.getContentPane().add(xmax);
	scale.getContentPane().add(cancel);
	scale.getContentPane().add(update);
    }

    /**
     * method create the message frame to inform the user when error occurs
     */
    public void createMessageFrame() {
	message = new JFrame("Error");
	//	message.setSize(dim);
	
	errortext = new JTextArea();
	errortext.setFont(newFont);
	errortext.setLineWrap(true);
	//	errortext.setHorizontalAlignment(JTextArea.CENTER);
	Dimension dim = new Dimension(200, 100);
	errortext.setPreferredSize(dim);

	msgdismiss = new JButton("Dismiss");
	msgdismiss.addActionListener(this);

	//	GridLayout layout = new GridLayout(2,1);
	//	message.getContentPane().setLayout(layout);
	message.getContentPane().add(errortext, BorderLayout.CENTER);
	message.getContentPane().add(msgdismiss, BorderLayout.SOUTH);
    }

    /**
     * method create the help frame to show the help files
     */
    public void createHelpFrame() {
	// create the frame for help message
	//
	helpframe = new JFrame("Help");

	// create the textbox to view the help message
	helptext = new JTextArea();
	helptext.setLineWrap(true);
	helptext.setWrapStyleWord(true);
	helptext.setEditable(false);

	// make the scrollpane for helpbox
	//
	scrollPane = new JScrollPane(helptext);
	scrollPane.setSize(new Dimension(400, 400));
	scrollPane.setPreferredSize(new Dimension(400, 400));
	scrollPane.setMinimumSize(new Dimension(400, 400));
	scrollPane.setMaximumSize(new Dimension(400, 400));
	
	// create the dismiss button for the text area
	//
	helpdismiss = new JButton("Dismiss");
	helpdismiss.setActionCommand("Done");
	helpdismiss.addActionListener(this);

	helpframe.getContentPane().add(scrollPane, BorderLayout.CENTER);
	helpframe.getContentPane().add(helpdismiss, BorderLayout.SOUTH);
    }

    /**
     * method listens for actions taking place on text only menu items
     *
     * @param e handles events fired by the action listeners
     *
     * @throws MalformedURLException thrown if URL cannot be loaded
     */
    public void actionPerformed(ActionEvent e) {
	
	// get the source of the action that just occured
	//
	String source = e.getActionCommand();

	// depending on the source of the event take the appropriate action
	//
	if (source.equals("From File")) {
	    urlframe.pack();
	    urlframe.setVisible(true);
	}
	else if (source.equals("URL Submit") ) {
	    urlframe.setVisible(false);
	    try {
		signaladdress = new URL(urlstring.getText());
	    }
	    catch (MalformedURLException err) {
		errortext.setText("Error open file: " + urlstring.getText());
		message.pack();
		message.setVisible(true);
		return;
	    }
	    gridmatch.setSignal(signaladdress);
	    testspec.attachData(gridmatch.getSignal());
	    testspec.changeTitle("User Defined Signal");
	    testspec.reFresh();
	    output.detachData();
	    output.reFresh();
	    testenergy.attachData(gridmatch.getSignal());
	    testenergy.repaint();
	    
	}
	else if (source.equals("URL Cancel")) {
	    urlframe.setVisible(false);
	}
	else if (source.equals("Select Signal")) {
	    signalSource.pack();
	    signalSource.setVisible(true);
	}
	/* need to modify these later.... */
	else if (source.equals("One")) {
	    setTemplate(0);
	} 
	else if (source.equals("Two")) {
	    setTemplate(1);
	}
	else if (source.equals("Three")) {
	    setTemplate(2);
	}
	else if (source.equals("Four")) {
	    setTemplate(3);
	}
	else if (source.equals("Five")) {
	    setTemplate(4);
	}
	else if (source.equals("Six")) {
	    setTemplate(5);
	}
	else if (source.equals("Seven")) {
	    setTemplate(6);
	}
	else if (source.equals("Eight")) {
	    setTemplate(7);
	}
	else if (source.equals("Nine")) {
	    setTemplate(8);
	}
	else if (source.equals("Zero")) {
	    setTemplate(9);
	}
	else if (source.equals("Oh")) {
	    setTemplate(10);
	}
	else if (source.equals("One - Male")) {
	    setSignal(0);
	} 
	else if (source.equals("Two - Male")) {
	    setSignal(1);
	} 
	else if (source.equals("Three - Male")) {
	    setSignal(2);
	} 
	else if (source.equals("Four - Male")) {
	    setSignal(3);
	} 
	else if (source.equals("Five - Male")) {
	    setSignal(4);
	} 
	else if (source.equals("Six - Male")) {
	    setSignal(5);
	} 
	else if (source.equals("Seven - Male")) {
	    setSignal(6);
	} 
	else if (source.equals("Eight - Male")) {
	    setSignal(7);
	} 
	else if (source.equals("Nine - Male")) {
	    setSignal(8);
	} 
	else if (source.equals("Zero - Male")) {
	    setSignal(9);
	} 
	else if (source.equals("Oh - Male")) {
	    setSignal(10);
	} 
	else if (source.equals("None")) {
	    gridmatch.setSlope(5.0);
	}
	else if (source.equals("Slope 2 - 1/2")) {
	    gridmatch.setSlope(2.0);
	}
	else if (source.equals("Slope 3 - 1/3")) {
	    gridmatch.setSlope(3.0);
	}
	else if (source.equals("Euclidean Distance")) {
	    gridmatch.setDistance("Euclidean");
	}

	else if (source.equals("Itakura Distance")) {
	    gridmatch.setDistance("Itakura");
	}

	else if (source.equals("Absolute Distance")) {
	    gridmatch.setDistance("Absolute");
	}

	else if (source.equals("Play")) {
	    URL url = gridmatch.getAudioFile();
	    if(url != null) {
		AudioClip clip = context.getAudioClip(url);
		clip.play();
	    }
	}

	else if (source.equals("Compute")) {
	    
	    if (gridmatch.getSignal() == null) {
		errortext.setText("Please select test signal first");
		message.pack();
		message.setVisible(true);
		return;
	    }
	    
	    if (gridmatch.getReference() == null) {
		errortext.setText("Please select model first");
		message.pack();
		message.setVisible(true);
		return;
	    }
	    
	    gridmatch.setRefIndex(0);
	    gridmatch.lefttorightMatch();
	    output.attachData(gridmatch.pathRecord[0], 
			      gridmatch.bestPathLength());
	    output.setYMax(gridmatch.getReference().getLength() - 1);
	    output.setYMin(0);
	    output.reFresh();
	    refspec.attachData(gridmatch.getReference());
	    refspec.reFresh();
	    testspec.attachData(gridmatch.getSignal());
	    testspec.reFresh();
	    warpedspec.attachData(gridmatch.getReference());
	    warpedspec.attachPath(gridmatch.pathRecord[0], 
				  gridmatch.bestPathLength());
	    warpedspec.reFresh();
	    refenergy.attachData(gridmatch.getReference());
	    refenergy.reFresh();
	    testenergy.attachData(gridmatch.getSignal());
	    testenergy.reFresh();
	    process.textArea.append("Compute result:\n" +
	    "    *Distance: " + gridmatch.getDistanceType() + newline +
	    "    *Slope: " + gridmatch.getSlope() + newline +
	    "    *Distance between selected signal and template is: " + myFormat.format(gridmatch.totalCost[0])+ "\n");
	    process.textArea.append("    *Accumulated distance at each node of final path: \n");
	    for (int i=0; i<gridmatch.getSignal().getLength(); i++) {
		process.textArea.append("        node["+i+"]["
					+gridmatch.pathRecord[0][i]+"]:"+"\t" +
				myFormat.format(gridmatch.costRecord[i])
					+"\n");
	    }

	    process.textArea.append("-----------------------------------\n");
	}
	else if (source.equals("Recognize")) {
	    if (gridmatch.getSignal() == null) {
 		errortext.setText("Please select signal first");
	 	message.pack();
		message.setVisible(true);
 		return;
	    }
	    gridmatch.bestMatch();
	    refspec.attachData(gridmatch.getReference());
	    refspec.changeTitle(Constants.models[gridmatch.recogResult]);
	    refspec.reFresh();

	    refenergy.attachData(gridmatch.getReference());
	    refenergy.repaint();

	    output.attachData(gridmatch.pathRecord[gridmatch.recogResult],
			      gridmatch.bestPathLength());
	    output.setYMax(gridmatch.getReference().getLength() -1);
	    output.setYMin(0);
	    output.reFresh();

	    testspec.attachData(gridmatch.getSignal());
	    testspec.reFresh();

	    warpedspec.attachData(gridmatch.getReference());
	    warpedspec.attachPath(gridmatch.pathRecord[gridmatch.recogResult],
				  gridmatch.bestPathLength());
	    warpedspec.reFresh();

	    process.textArea.append("Recognize result:\n" +
	    "    *Distance: " + gridmatch.getDistanceType() + newline +
	    "    *Slope: " + gridmatch.getSlope() + newline +
	    "    *The signal is recognized as" + Constants.words[gridmatch.recogResult] + newline + 
	    "    *Distance between this signal templates are:" + newline);
	    
	    for (int i=0; i<11; i++) {
		process.textArea.append("         template " + 
		    Constants.words[i] + ": " +
		    myFormat.format(gridmatch.totalCost[i]) + newline);
		
	    }
	    process.textArea.append("------------------------------------\n");
	}
	else if (source.equals("Clear")) {
	    gridmatch.setSignal(null);
	    gridmatch.setReference(null);
	    output.clearPanel();
	    output.detachData();
	    output.reFresh();
	    refspec.attachData(null);
	    refspec.reFresh();
	    testspec.attachData(null);
	    testspec.reFresh();
	    warpedspec.attachData(null);
	    warpedspec.reFresh();
	    process.textArea.setText(null);
	    refenergy.attachData(null);
	    refenergy.repaint();
	    testenergy.attachData(null);
	    testenergy.repaint();
	    
	    
	}
	else if (source.equals("Clear Description")) {
	    process.textArea.setText(null);
	}
	else if (source.equals("Scaling")) {
	    scale.pack();
	    scale.setVisible(true);
	}    
	else if (source.equals("Cancel")) {
	    scale.setVisible(false);
	}
	else if (source.equals("Dismiss")) {
	    errortext.setText(null);
	    message.setVisible(false);
	}
	else if (source.equals("Update")) {
	    int newxmax, newxmin;
	    newxmax = Integer.valueOf(xmax.getText()).intValue();
	    newxmin = Integer.valueOf(xmin.getText()).intValue();
	    scale.setVisible(false);
	    if (newxmax <= newxmin) {
		errortext.setText("XMax must be greater than XMin!");
		message.pack();
		message.setVisible(true);
		return;
	    }
	    else if ((newxmax > gridmatch.getSignal().getLength()) || 
		     (newxmin < 0)) {
		errortext.setText("Input number out of range. Please try again");
		message.pack();
		message.setVisible(true);
		return;
	    }
	    output.setXMax(newxmax);
	    output.updateYMax();
	    output.setXMin(newxmin);
	    output.updateYMin();
	    output.reFresh();
	    refspec.setXMax(output.getYMax());
	    refspec.setXMin(output.getYMin());
	    refspec.reFresh();
	    testspec.setXMax(output.getXMax());
	    testspec.setXMin(output.getXMin());
	    testspec.reFresh();
	    warpedspec.setXMax(output.getXMax());
	    warpedspec.setXMin(output.getXMin());
	    warpedspec.reFresh();
	    testenergy.setXMax(output.getXMax());
	    testenergy.setXMin(output.getXMin());
	    testenergy.repaint();
	    refenergy.setXMax(output.getYMax());
	    refenergy.setXMin(output.getYMin());
	    refenergy.repaint();
	    waveform.setXMax(output.getXMax());
	    waveform.setXMin(output.getXMin());
	    waveform.reFresh();
	}
	else if (source.equals("Reset")) {
	    if (gridmatch.getSignal() != null) {
		output.setXMax(gridmatch.getSignal().getLength() - 1);
		output.setXMin(0);
		output.reFresh();
		testspec.recoverPanel();
		testenergy.recoverPanel();
		waveform.recoverPanel();
	    }
	    if (gridmatch.getReference() != null) {
		output.setYMax(gridmatch.getReference().getLength() - 1);
		output.setYMin(0);
		output.reFresh();
		refspec.recoverPanel();
		refenergy.recoverPanel();
	    }
	    warpedspec.recoverPanel();
	    warpedspec.reFresh();
	}
	else if (source.equals("Release Notes")) {
	    // clear the text in the text area
	    //
	    helptext.setText("");

	    // read the help file contents and append them to the text area
	    //
	    try{
		// get the URL component
		//
		helpfile = (URL)urlvec.elementAt(0);
		// open a file descriptor to read the help file
		//
		InputStreamReader rd = new InputStreamReader(helpfile.openStream());
		// initialize the buffer to read the contents of the help file
		//
		buff = new BufferedReader(rd);
		// read each line for the file
		//
		while((document = buff.readLine()) != null) {
		    helptext.append(document + newline);
		}
	    }
	    catch(IOException ioe){}
	    helpframe.pack();
	    helpframe.setVisible(true);
	}
	else if (source.equals("Information and Support")) {
	    // clear the text in the text area
	    //
	    helptext.setText("");

	    // read the help file contents and append them to the text area
	    //
	    try{
		// get the URL component
		//
		helpfile = (URL)urlvec.elementAt(1);
		// open a file descriptor to read the help file
		//
		InputStreamReader rd = new InputStreamReader(helpfile.openStream());
		// initialize the buffer to read the contents of the help file
		//
		buff = new BufferedReader(rd);
		// read each line for the file
		//
		while((document = buff.readLine()) != null) {
		    helptext.append(document + newline);
		}
	    }
	    catch(IOException ioe){}
	    helpframe.pack();
	    helpframe.setVisible(true);
	}

	else if (source.equals("Applet Overview")) {
	    // clear the text in the text area
	    //
	    helptext.setText("");

	    // read the help file contents and append them to the text area
	    //
	    try{
		// get the URL component
		//
		helpfile = (URL)urlvec.elementAt(2);
		// open a file descriptor to read the help file
		//
		InputStreamReader rd = new InputStreamReader(helpfile.openStream());
		// initialize the buffer to read the contents of the help file
		//
		buff = new BufferedReader(rd);
		// read each line for the file
		//
		while((document = buff.readLine()) != null) {
		    helptext.append(document + newline);
		}
	    }
	    catch(IOException ioe){}
	    helpframe.pack();
	    helpframe.setVisible(true);
	}
	else if (source.equals("Applet Tutorial")) {
	    // show the tutorial in another browser
	    //
	    try {
		// get the URL component
		//
		helpfile = (URL)urlvec.elementAt(3);
		
		// display the url on a seperate browser
		//
		context.showDocument(helpfile, "_blank");
	    }
	    catch(Exception ae) {}
	}
	else if(source.equals("About Plug-ins")) {

	    // clear the text in the text area
	    //
	    helptext.setText("");
            
	    // read the help file contents and append them to the text area
	    //
	    try { 

		// get the URL component
		//
		helpfile = (URL)urlvec.elementAt(4);

		// open a file descriptor to read the help file
		//
		InputStreamReader rd = new InputStreamReader(helpfile.openStream());

		// initialize the buffer to read the contents of the help file
		//
		buff = new BufferedReader(rd); 

		// read each line fo the file
		//
		while((document = buff.readLine()) != null) { 

		    // add the help message line to the text area
		    //
		    helptext.append(document + newline);
		}
	    } 
	    catch(IOException ioe) { }

            // display the text area and dismiss button
            //
            helpframe.pack();
            helpframe.setVisible(true);
	}
	else if (source.equals("Done")) {
	    helpframe.setVisible(false);
	}
    }
    
    private void setTemplate(int a) {
	gridmatch.setReference(a);
	if (gridmatch.getReference() == null) 
	    return;
	else {
	    refspec.attachData(gridmatch.getReference());
	    refspec.changeTitle(Constants.models[a]);
	    refspec.reFresh();
	    output.detachData();
	    output.setYMax(gridmatch.getReference().getLength()-1);
	    output.setYMin(0);
	    output.reFresh();
	    refenergy.attachData(gridmatch.getReference());
	    refenergy.repaint();
	}
    }
    private void setSignal(int a) {
	gridmatch.setSignal(a);
	if (gridmatch.getSignal() == null) 
	    return;
	else {
	    testspec.attachData(gridmatch.getSignal());
	    testspec.changeTitle(Constants.testsignals[a]);
	    testspec.reFresh();
	    output.detachData();
	    output.setXMax(gridmatch.getSignal().getLength() - 1);
	    output.setXMin(0);
	    output.reFresh();
	    testenergy.attachData(gridmatch.getSignal());
	    testenergy.repaint();
	    waveform.setSignal(gridmatch.getRawURL());
	    if (waveform.getLength() == 0) {
		setErrorInfo("Error opening waveform file. This doesn't matter if the LPC file can be input properly");
	    }
	}
    }

    /**
     * This method sets the error info
     *
     * @param a String value of error message
     */
    public void setErrorInfo(String a) {
	errortext.setText(a);
	message.pack();
	message.setVisible(true);
    }
    
}
