// file: OutputPanel
//

// import necessary java libraries
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

/**
 * OutputPanel is used as a frame of a DrawingPanel.
 *
 * hierarchy: JPanel -> SubPanel -> OutputPanel
 */
public class OutputPanel extends SubPanel {

    // *****************************************************************
    //
    // declare private variables and components
    //
    // *****************************************************************
    // canvas to draw the grid path
    private DrawingPanel canvas;

    // name of this panel
    private String title;

    // font of name label
    private Font currentFont = getFont();   // fonts for the title border
    private Font newFont = new Font(currentFont.getName(), 
				    currentFont.getStyle(), 12);


    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * default class constructor
     */
    public OutputPanel() {
	super();

	canvas = new DrawingPanel();
	
	// create the title border using input argument
	title = new String("Grid Path: ");
	Border titleborder = BorderFactory.createTitledBorder(
			     BorderFactory.createLineBorder(Color.black),
			     title,
			     TitledBorder.LEFT,
			     TitledBorder.ABOVE_TOP,
			     newFont,
			     Color.black);
	setBorder(titleborder);

	// add canvas to this panel
	addComponents();
    }

    // *****************************************************************
    // 
    // declare class methods
    //
    // *****************************************************************

    /**
     * add canvas to the panel
     */
    public void addComponents() {
	constrain(this, canvas, 0, 0,
		  GridBagConstraints.REMAINDER, 1,
		  GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
		  1, 1, 0, 0, 0, 0);
    }

    /**
     * repaints the canvas
     */
    public void reFresh() {
	super.repaint();
	canvas.repaint();
    }

    // method: attachData
    //
    // arguments: int d[], int len
    // return   : none
    /**
     * send path and its length to canvas
     *
     * @param d data to be used
     * @param len length of data
     */
    public void attachData(int d[], int len) {
	canvas.attachData(d, len);
    }

    /**
     * Method assigns null to data variable, 0 to datalenghth, and
     * false to dataAttached flag 
     */
    public void detachData() {
	canvas.detachData();
    }

    /**
     * clears the whole canvas
     */
    public void clearPanel() {
	canvas.clearPanel();
    }

    /**
     * draw the original path after rest
     */
    public void recoverPanel() {
	canvas.recoverPanel();
    }

    /**
     * access method to get xMax
     *
     * @return xMax int value
     */
    public int getXMax() {
	return canvas.getXMax();
    }

    /**
     * access method to get xMin
     *
     * @return xMin int value
     */
    public int getXMin() {
	return canvas.getXMin();
    }

    /**
     * access method to get yMax
     *
     * @return yMax int value
     */
    public int getYMax() {
	return canvas.getYMax();
    }

    /**
     * access method to get yMin
     *
     * @return yMin int value
     */
    public int getYMin() {
	return canvas.getYMin();
    }

    /**
     * access method to get XMargin
     *
     * @return XMargin int value
     */
    public int getXMargin() {
	return canvas.getXMargin();
    }

    /**
     * access method to get YMargin
     *
     * @return YMargin int value
     */
    public int getYMargin() {
	return canvas.getYMargin();
    }

    /**
     * set max value of canvas drawing range for x
     *
     * @param a int value to set
     */
    public void setXMax(int a) {
	canvas.setXMax(a);
    }

    /**
     * set min value of canvas drawing range for x
     *
     * @param a int value to set
     */
    public void setXMin(int a) {
	canvas.setXMin(a);
    }
    
    /**
     * set max value of canvas drawing range for y
     *
     * @param a int value to set
     */
    public void setYMax(int a) {
	canvas.setYMax(a);
    }

    /**
     * set min value of canvas drawing range for y
     *
     * @param a int value to set
     */
    public void setYMin(int a) {
	canvas.setYMin(a);
    }

    /**
     * update canvas drawing range of y according to the range of x
     */
    public void updateYMax() {
	canvas.updateYMax();
    }

    /**
     * update canvas drawing range of y according to the range of x
     */
    public void updateYMin() {
	canvas.updateYMin();
    }

    // method: setDataFlag
    //
    // arguments: boolean a
    // return   : none
    //
    // 
    // set the drawing flag for canvas
    /*
    public void setDataFlag(boolean a) {
	canvas.dataAttached = a;
    }
     */
}

// end of file
