// file: SignalPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.net.*;

/**
 * SignalPanel is used as a frame of a SpectrumPanel.
 *
 * hierarchy: JPanel -> SubPanel -> signalPanel
 */
public class SignalPanel extends SubPanel {

    //**********************************************************************
    //
    // declare global variables
    //
    //**********************************************************************

    // SpectrumPanel to draw the spectrogram for attached lpc coded signal
    //
    private SpectrumPanel canvas;

    // name of the signal
    //
    private String title;
    
    private TitledBorder titleborder;

    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * The default constructor initializes variables and components.
     *
     * @param str title to be used
     */
    public SignalPanel(String str) {

	// initialize the canvas to draw spectrogram
	//
	canvas = new SpectrumPanel();

	// initialize the titles of the objects
	//
	title = str;

	// create and set border for panel
	//
	Font currentFont = getFont();
	Font newFont = new Font(currentFont.getName(), 
					currentFont.getStyle(), 12);
	titleborder = new TitledBorder
	    (BorderFactory.createLineBorder(Color.black),
	     title,
	     TitledBorder.LEFT,
	     TitledBorder.ABOVE_TOP,
	     newFont,
	     Color.black);
	setBorder(titleborder);

	// add canvas to panel
	//
	addComponents();
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************
    
    // method: addComponents
    //
    // arguments: none
    // return   : none
    //
    // add components to panel
    //
    private void addComponents() {
	constrain(this, canvas, 0, 0,
		  GridBagConstraints.REMAINDER, 1,
		  GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
		  1, 1, 0, 0, 0, 0);
    }

    /**
     * This method changes title according to attached signal.
     *
     * @param a title to be set
     */
    public void changeTitle(String a) {
	titleborder.setTitle(a);
    }

    /**
     * This method sets drawing mode for canvas.
     *
     * @param a String value of drawing mode to be set
     */
    public void setDrawMode(String a) {
	canvas.setDrawMode(a);
    }

    /**
     * This method recovers the canvas.
     */
    public void recoverPanel() {
	canvas.recoverPanel();
    }

    /**
     * This method repaints the canvas.
     */
    public void reFresh() {
	super.repaint();
	canvas.repaint();
    }

    /**
     * This method sends lpcData to canvas.
     *
     * @param d lpcData object to be attached
     */
    public void attachData(lpcData d) {
	canvas.attachData(d);
    }

    /**
     * This method clears the canvas
     */
    public void clearPanel() {
	canvas.clearPanel();
    }

    /**
     * This method sets max value of x for canvas.
     *
     * @param a int value to set
     */
    public void setXMax(int a) {
	canvas.setXMax(a);
    }

    /**
     * This method sets min value of x for canvas.
     *
     * @param a int value to set
     */
    public void setXMin(int a) {
	canvas.setXMin(a);
    }

    /**
     * This method sends warping path to canvas to draw spectrogram for
     * warped signal
     *
     * @param path   array for path info
     * @param length path length
     */
    public void attachPath(int path[], int length) {

	canvas.attachPath(path, length);
    }
}

// end of file
