// file: SubPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

/**
 * creates a panel with layout type of GridBagLayout.
 * 
 * hierarchy: JPanel->SubPanel
 */
public class SubPanel extends JPanel {
    
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    public GridBagLayout gridbag = new GridBagLayout();
    
    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
    
    /**
     * sets size, background properties, layout manager
     */
    public SubPanel() {
	
	this.setLayout(gridbag);
	//	this.setBorder(BorderFactory.createEtchedBorder());
    }
    
    /**
     * This method creates the constraints of a component in a container
     * and then adds the component to the container.  it is based on 
     * code from "Java in a Nutshell" by David Flanagan. published by
     * O'Reilly and Associates incorporated.
     *
     * @param container    container to place the component
     * @param component    the component to be constrained
     * @param gridX        horizontal position of component
     * @param gridY        vertical position of component
     * @param gridWidth    number of horizontal grid spots to take
     * @param gridHeight   number of vertical grid spots to take
     * @param fill         directions to fill when component space is resized
     * @param anchor       anchor position when component space is resized
     * @param weightX      relative amount of horizontal component
     *                     should take when its space is resized
     * @param weightY      relative amount of vertical space component
     *                     should take when its space is resized
     * @param top          amount of inset space on top
     * @param left         amount of inset space on left
     * @param bottom       amount of inset space on bottom
     * @param right        amount of inset space on right
     * 
     * @return true
     */
    public boolean constrain(Container container, Component component, 
			     int gridX, int gridY, int gridWidth, 
			     int gridHeight, int fill, int anchor, 
			     double weightX, double weightY, int top,
			     int left, int bottom, int right) {
	
	// set the constraints based on the input variables
	//
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = gridX;
	c.gridy = gridY;
	c.gridwidth = gridWidth;
	c.gridheight = gridHeight;
	c.fill = fill;
	c.anchor = anchor;
	c.weightx = weightX;
	c.weighty = weightY;
	
	// if any of the buffer insets are greater than zero, create a new
	// Insets object
	//
	if (top + bottom + left + right > 0)
	    c.insets = new Insets(top, left, bottom, right);
	
	// set the constraints of the component
	//
	((GridBagLayout) container.getLayout()).setConstraints(component, c);
	
	// add the component to the container
	//
	container.add(component);
	
	// exit gracefully
	//
	return true;
    }
}

// 
// end of file





