import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.net.*;

/**
 * class that handles the wave panel sub panel
 */
public class WavePanel extends SubPanel {

    //**********************************************************************
    //
    // declare private variables
    //
    //**********************************************************************

    private WaveformPanel canvas;

    private String title;
    
    // fonts for the title border
    //
    private Font currentFont = getFont();  
    private Font newFont = new Font(currentFont.getName(), 
				    currentFont.getStyle(), 12);
    private TitledBorder titleborder;


    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * default class constructor
     *
     * @param str String to set the title
     */
    public WavePanel(String str) {
   
        canvas = new WaveformPanel();

	title = str;

	titleborder = new TitledBorder
	    (BorderFactory.createLineBorder(Color.black),
	     title,
	     TitledBorder.LEFT,
	     TitledBorder.ABOVE_TOP,
	     newFont,
	     Color.black);

	setBorder(titleborder);
	addComponents();
	
    }

    /**
     * adds components to the panel
     */
    private void addComponents() {
	constrain(this, canvas, 0, 0,
		  GridBagConstraints.REMAINDER, 1,
		  GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
		  1, 1, 0, 0, 0, 0);
    }

    /**
     * This method sets the title.
     *
     * @param a String value to be set
     */
    public void changeTitle(String a) {
	titleborder.setTitle(a);
    }

    /**
     * This method repaints the canvas.
     */
    public void reFresh() {
	super.repaint();
	canvas.repaint();
    }

    /**
     * This method recovers the panel.
     */
    public void recoverPanel(){
	canvas.recoverPanel();
    }

    /**
     * This method sets the signal given the URL
     */
    public void setSignal(URL fileurl) {
	canvas.setSignal(fileurl);
    }

    /**
     * This method sets xMax
     *
     * @param a int value to be set
     */
    public void setXMax(int a) {
	canvas.setXMax(a);
    }

    /**
     * This method sets xMin
     *
     * @param a int value to be set
     */
    public void setXMin(int a) {
	canvas.setXMin(a);
    }

    /**
     * This method gets the canvas length
     *
     * @return canvas length
     */
    public int getLength() {
	return canvas.getLength();
    }
}
