// file: dtwApplet.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.applet.*;
import java.net.*;
import javax.swing.border.*;
 

/**
 * dtwApplet is the main driver program that extend JApplet and is the 
 * class called when the applet is loaded, which inturn initialize all
 * other objects and components need to run the applet
 *
 * hierarchy: JApplet->dtwApplet
 */
public class dtwApplet extends JApplet {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    // declare global variables
    //
    URL url;
    Vector<URL> urlvec = new Vector<URL>(5,1);

    // declare components
    //
    private MainMenu control;
    private match gm;
    private OutputPanel output;
    private SignalPanel refspectrum;
    private SignalPanel testspectrum;
    private SignalPanel warpedspectrum;
    private EnergyPanel refenergy;
    private EnergyPanel testenergy;
    private WavePanel waveform;
    private ProcessBox process;
    private inputSignals signalSource;
    
    private JPanel contentPane = new JPanel();


    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
   
    /**
     * class default constructor
     */
    public dtwApplet() {
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************
    
    /**
     * this method is called when the applet is started
     */
    public void init() {
	super.init();

	gm = new match(this);

	refspectrum = new SignalPanel("Model:");
	Dimension d1 = new Dimension(100, 200);
 	refspectrum.setPreferredSize(d1);
	refspectrum.setMaximumSize(d1);
	refspectrum.setMinimumSize(d1);
	refspectrum.setDrawMode("Vertical");
	

	testspectrum = new SignalPanel("Test Signal:");
	Dimension d2 = new Dimension(400, 100);
 	testspectrum.setPreferredSize(d2);
	testspectrum.setMaximumSize(d2);
	testspectrum.setMinimumSize(d2);
	testspectrum.setDrawMode("Horizontal");

	warpedspectrum = new SignalPanel("Warped Output:");
	Dimension d3 = new Dimension(400, 100);
	warpedspectrum.setPreferredSize(d3);
	warpedspectrum.setMaximumSize(d3);
	warpedspectrum.setMinimumSize(d3);
	warpedspectrum.setDrawMode("Warped");

	output = new OutputPanel();
	Dimension d4 = new Dimension(400, 200);
 	output.setPreferredSize(d4);
	output.setMaximumSize(d4);
	output.setMinimumSize(d4);

	refenergy = new EnergyPanel("Energy");
	Dimension d5 = new Dimension(100, 200);
	refenergy.setPreferredSize(d5);
	refenergy.setMaximumSize(d5);
	refenergy.setMinimumSize(d5);
	refenergy.setDrawMode("Vertical");

	testenergy = new EnergyPanel("Energy");
	Dimension d6 = new Dimension(400, 100);
	testenergy.setPreferredSize(d6);
	testenergy.setMaximumSize(d6);
	testenergy.setMinimumSize(d6);
	testenergy.setDrawMode("Horizontal");

	process = new ProcessBox();
 	Dimension d7 = new Dimension(200, 400);
	process.setPreferredSize(d7);
	process.setMaximumSize(d7);
	process.setMinimumSize(d7);

	waveform = new WavePanel("Signal");
	Dimension dd = new Dimension(400, 100);
	waveform.setPreferredSize(dd);
	waveform.setMaximumSize(dd);
	waveform.setMinimumSize(dd);
	

	signalSource = new inputSignals(this, gm, testspectrum, testenergy,
					output, "../data/signal.list"); 


	getRootPane().putClientProperty("defeatSystemEventQueueCheck",
					Boolean.TRUE);

	// declare components to read the help files
	//
	try {
	    url = new URL(getCodeBase() + "doc/help_release.txt");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/help_info_support.txt");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/help_overview.txt");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/tutorial.html");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/help_plugins.txt");
	    urlvec.addElement(url);
	}
	catch(MalformedURLException me) { }
	
	// get the applet context
	//
	AppletContext context = getAppletContext();

	// initialize the menu bar object
	//
	control = new MainMenu(gm, output, process, 
			       refspectrum, testspectrum, warpedspectrum,
			       refenergy, testenergy, waveform,
			       signalSource, context, urlvec);
	signalSource.setMenu(control);
	Dimension d8 = new Dimension(600, 40);
	control.setPreferredSize(d8);
	control.setMaximumSize(d8);
	control.setMinimumSize(d8);

	gm.setMenu(control);
	gm.initTemplates();

	// add the components
	//
	addComponents();
    }

    // method: addComponents
    //
    // arguments: none
    // return   : none
    //
    // this method is called by init() to add components to the applet
    //
    private void addComponents() {

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();

	this.setContentPane(contentPane);
	contentPane.setLayout(gb);


	control.constrain(contentPane, control, 0, 0,
			  GridBagConstraints.REMAINDER, 1,
			  GridBagConstraints.BOTH,
			  GridBagConstraints.NORTHWEST,
			  1, 0.1, 2, 2, 2, 2);

	refenergy.constrain(contentPane, refenergy, 1, 1, 1, 1,
			    GridBagConstraints.BOTH,
			    GridBagConstraints.NORTHWEST,
			    1, 1, 0, 0, 0, 0);

	refspectrum.constrain(contentPane, refspectrum, 0, 1, 1, 1,
			      GridBagConstraints.BOTH,
			      GridBagConstraints.NORTHWEST,
			      1, 1, 0, 0, 0, 0);
	
	output.constrain(contentPane, output, 2, 1, 1, 1,
			 GridBagConstraints.BOTH,
			 GridBagConstraints.NORTHWEST,
			 1, 1, 0, 0, 0, 0);

	testspectrum.constrain(contentPane, testspectrum, 2, 2, 1, 1,
			       GridBagConstraints.BOTH,
			       GridBagConstraints.NORTHWEST,
			       1, 1, 0, 0, 0, 0);
	
	testenergy.constrain(contentPane, testenergy, 2, 3, 1, 1,
			     GridBagConstraints.BOTH,
			     GridBagConstraints.NORTHWEST,
			     1, 1, 0, 0, 0, 0);
	
	waveform.constrain(contentPane, waveform, 2, 4, 1, 1,
			   GridBagConstraints.BOTH,
			   GridBagConstraints.NORTHWEST,
			   1, 1, 0, 0, 0, 0);

	warpedspectrum.constrain(contentPane, warpedspectrum, 2, 5, 1, 1,
				 GridBagConstraints.BOTH,
				 GridBagConstraints.NORTHWEST,
				 1, 1, 0, 0, 0, 0);


	process.constrain(contentPane, process, 0, 2, 2, 4,
			  GridBagConstraints.BOTH,
			  GridBagConstraints.NORTHWEST,
			  1, 1, 0, 0, 0, 0);
    }
}

//
// end of file			 
