import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.applet.AudioClip;

/**
 * Frame that handles the input signals
 */
public class inputSignals extends JFrame 
    implements ActionListener {
    private URL signals[];
    private URL audios[];
    private int selectedID;
    private JList listBox;
    private JScrollPane listScroll;
    private JButton playButton;
    private JButton selectButton;
    private JButton cancelButton;
    private JPanel buttonPane;
    private dtwApplet applet;
    private match gridmatch;
    private SignalPanel specpanel;
    private EnergyPanel energypanel;
    private OutputPanel output;
    private int maxArray;

    private MainMenu menu;

    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * default class constructor
     *
     * @param ap dtwApplet object to be initialized
     * @param gm match object to be initialized
     * @param sp SignalPanel object used
     * @param energy EnergyPanel object to be used
     * @param ot OutputPanel object to be used
     * @param signalListFile String of signal files
     *
     * @throws MalformedURLException thrown if the applet code base cannot
     *                               be found
     * @throws IOException thrown if the filestream cannot be initialized
     */
    public inputSignals(dtwApplet ap, 
		 match gm, SignalPanel sp, EnergyPanel energy,
		 OutputPanel ot, String signalListFile) { 
	super();

	applet = ap;
	gridmatch = gm;
	specpanel = sp;
	energypanel = energy;
	output = ot;
	selectedID = -1;

	InputStreamReader listfile = null;
	BufferedReader reader = null;
	URL listURL = null;


	Dimension d = new Dimension (300, 300);
	this.setSize(d);

	try {
	    listURL=new URL(applet.getCodeBase(), signalListFile);
	    

	}
	catch (MalformedURLException e) {
	    System.out.println(e);
	    System.exit(1);
	}
	    

	try {
	    listfile = new InputStreamReader(listURL.openStream());
	    reader = new BufferedReader(listfile);
	   

	} 
	catch (IOException e) {
	    System.out.println(e);
	    System.exit(1);
	}


	
	String buf = "";
	Vector<String> vec = new Vector<String>(100);
	int idx = 0, begin, end;
	
	String str = "";

	
	// to count the number of whole file
	while(true){
	    try{
		buf = reader.readLine();
	       
	    } 	    
	    catch (IOException e) {
		System.out.println(e);
		break;
	    }
	   
	   if (buf == null){	
	   break; //done reading file
	    }
	   
	   idx++;    // count the number of file
	   
	}
	
	maxArray = idx;   //reading the maximum file number
	// set the exact number of array size for signals and audios based on counting the whole file
	signals = new URL[maxArray];
	audios = new URL[maxArray];

	while(true) {
	    try{
		buf = reader.readLine();
	       
	    } 
	    catch (IOException e) {
		System.out.println(e);
		break;
	    }


	    if(idx == maxArray) {
		break; //done reading file
	    }
	    
	    str = buf.trim();  // removes leading and ending blanks
	    begin = 0;
	    for(end=0;end<str.length();end++) 
		if(str.charAt(end) == ' ')
		    break;

	    vec.addElement(str.substring(begin, end));
	    
	    for(end++; end < str.length(); end++) 
		if(str.charAt(end) != ' ')
		    break;
	    begin = end;
	    for(;end<str.length();end++) {
		if(buf.charAt(end) == ' ')
		    break;
	    }

	    try {
		
		signals[idx] = new URL(applet.getCodeBase(), str.substring(begin,end));
	    }
	    catch (MalformedURLException e) {
		System.out.println(e);
		System.exit(1);
	    }

	     
	    for(end++; end < str.length(); end++) 
		if(buf.charAt(end) != ' ')
		    break;
	    
	    

	    try {
	

		audios[idx] = new URL(applet.getCodeBase(),
				      str.substring(begin, str.length()));
	    } 
	    catch (MalformedURLException e) {
		System.out.println(e);
		System.exit(1);
	    }

	    begin = end;
	    
	    idx++;
	    
	}

	listBox = new JList(vec);
	listBox.setSelectedIndex(0);
	listScroll = new JScrollPane(listBox);
	this.getContentPane().add(listScroll, BorderLayout.CENTER);
	playButton = new JButton("Play");
	selectButton = new JButton ("Select");
	cancelButton = new JButton ("Cancel");
	playButton.addActionListener(this);
	selectButton.addActionListener(this);
	cancelButton.addActionListener(this);	
	buttonPane = new JPanel();
	buttonPane.add(playButton);
	buttonPane.add(selectButton);
	buttonPane.add(cancelButton);
	this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
    }

    /**
     * Sets the main menu
     *
     * @param m MainMenu object to be set
     */
    public void setMenu(MainMenu m) {
	menu = m;
    }

    /**
     * Handles action to the input signal frame
     *
     * @param e ActionEvent object fired to call method
     */
    public void actionPerformed(ActionEvent e) {
	String source = e.getActionCommand();
	int i = listBox.getSelectedIndex();
	if(source.equals("Play")) {
	    AudioClip clip = applet.getAudioClip(audios[i]);
	    clip.play();
	}
	else if(source.equals("Select")) {
	    gridmatch.setSignal(signals[i]);
	    gridmatch.setAudioFile(audios[i]);
	    lpcData lpc = gridmatch.getSignal();
	    specpanel.attachData(lpc);
	    specpanel.changeTitle(listBox.getSelectedValue().toString());
	    specpanel.reFresh();
	    energypanel.attachData(lpc);
	    energypanel.repaint();
	    output.detachData();
	    output.reFresh();
	    this.setVisible(false);
	}
	else if(source.equals("Cancel")) {
	    this.setVisible(false);
	}
    }
}
