// file: lpcData.java
//

// import necessary java libraries
//
import java.lang.*;
import java.io.*;
import java.net.*;

/**
 * lpcData is used to represent a lpc coded speech signal.
 * it reads in the signal from ascii file, and stores signal in vectors
 * each vector reflects to a frame of speech signal.
 */
public class lpcData {
    
    //**********************************************************************
    //
    // declare global variables
    //
    //**********************************************************************

    /**
     * array to store lpc vectors
     * this applet using 12-order lpc coded signal
     */
    public double data[][] = new double[200][13];

    // length of signal
    //
    private int length;

    // location of original file
    //
    private URL url = null;

    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * This constructor opens file specified by input argument, read in
     * signal, and store in data[][].
     *
     * @param file URL of the file to be loaded
     */
    public lpcData(URL file) {
	url = file;
	loadData(url);
    }

    // *****************************************************************
    // 
    // declare class methods
    //
    // *****************************************************************

    /**
     * access method for length
     *
     * @return length of signal
     */
    public int getLength() {
	return length;
    }

    // method: loadData
    //
    // arguments: none
    // return   : none
    // 
    // read the ascii file of a lpc coded signal/template, fill in the data 
    // array, each frame is a vector of 13 elements of double number
    //
    private void loadData(URL signalURL) {

	// index for frame and element
	//
	int frameIndex, elementIndex;

	// index of first and last pointers of one ascii-form double 
	// number
	//
	int begin=0, end=0;

	// temp: string to store a line of chars
	// num:  string to store the chars for a double number
	//
	String temp = " " , num = " ";

	// input stream of signal file
	//
	InputStreamReader isr = null;
	BufferedReader br = null;

	// open the input file
	//
	try {
	    isr = new InputStreamReader(signalURL.openStream());
	    br = new BufferedReader(isr);
	}
	catch (IOException e) {
	    System.out.println(e);
	    data = null;
	    length = 0;
	    url = null;
	    return;
	}

	// skip the header
	//
	try {
	    br.readLine();
	    br.readLine();
	    br.readLine();
	    br.readLine();
	    br.readLine();
	}
	catch (IOException e) {
	    System.out.println(e);
	    data = null;
	    url = null;
	    length = 0;
	    return;
	}

	// initialize length and frame index
	//
	frameIndex = -1;
	length = -1;

	// each line read from the file would be one lpc vector.
	// elements in one vector are seperated by ","
	//
	while (true) {
	    try {

		// store the meaningful line in string temp
		//
		temp = br.readLine();

		// the line to seperate two vectors will be skipped
		//
		br.readLine();
	    }
	    catch (IOException e) {
		System.out.println(e);
		data = null;
		url = null;
		length = 0;
		return;
	    }

	    // touch the end of file
	    //
	    if (temp == null) break;

	    // increase frame index and signal length by 1
	    //
	    frameIndex += 1;
	    length +=1;

	    // initialize element index, starting and ending char
	    // pointer for an element
	    //
	    elementIndex = 0;
	    begin = 0;
	    end = 0;

	    // scan string temp, convert substring of temp flagged by
	    // (begin, end) as double number while end hits ',' or the last
	    // char of string temp 
	    //
	    while(true) {
		
		// scans to the last char of temp, need to convert substring
		// of temp, and prepare to read in next line
		//
		if (end == temp.length()) {
		    num = temp.substring(begin, end);
		    data[frameIndex][elementIndex] = 
			Double.valueOf(num).doubleValue();
		    break;
		}

		// pointer end hits a ',', need to convert substring of temp
		// reset pointer begin and end
		//
		if (temp.charAt(end) == ',') {
		    num = temp.substring(begin, end);
		    data[frameIndex][elementIndex] = 
			Double.valueOf(num).doubleValue();

		    // begin and end point to the first char of next element
		    //
		    end += 1;
		    begin = end;

		    // increase index of element by 1
		    //
		    elementIndex += 1;
		}
		end += 1;
	    }
	}

	// read in all frames. since frame index starts from 0, length 
	// should increase by 1
	//
	length++;

	// close input stream
	//
	try {
	    isr.close();
	    br.close();
	}
	catch(IOException e) {
	    System.out.println(e+"error close file");
	}
    }

}

// end of file
